﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Threading;
using Telerik.Windows.Data;

namespace Telerik.Windows.Controls.GridView.Scrolling
{
    public class CustomScrolling
    {
        private GridViewDataControl grid;

        public CustomScrolling(GridViewDataControl grid)
        {
            this.grid = grid;
        }

        public static readonly DependencyProperty DomainDataSourceProperty = DependencyProperty.RegisterAttached("DomainDataSource",
            typeof(DomainDataSource), typeof(CustomScrolling),
                new PropertyMetadata(new PropertyChangedCallback(OnDomainDataSourcePropertyChanged)));

        public static void SetDomainDataSource(DependencyObject dependencyObject, DomainDataSource enabled)
        {
            dependencyObject.SetValue(DomainDataSourceProperty, enabled);
        }

        public static DomainDataSource GetDomainDataSource(DependencyObject dependencyObject)
        {
            return (DomainDataSource)dependencyObject.GetValue(DomainDataSourceProperty);
        }

        private static void OnDomainDataSourcePropertyChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            var grid = (GridViewDataControl)dependencyObject;
            var source = (DomainDataSource)e.NewValue;

            grid.SizeChanged -= SizeChanged;
            
            if (source != null)
            {
                grid.SizeChanged += SizeChanged;
            }
        }

        static void SizeChanged(object sender, SizeChangedEventArgs e)
        {
            var grid = (GridViewDataControl)sender;
            var source = CustomScrolling.GetDomainDataSource(grid);

            var sw = grid.ChildrenOfType<GridViewScrollViewer>().FirstOrDefault();
            if (sw != null)
            {
                sw.IsCustomScrollingEnabled = true;

                var vsb = sw.ChildrenOfType<ScrollBar>().Where(s => s.Orientation == Orientation.Vertical).FirstOrDefault();
                if (vsb != null)
                {
                    ScrollViewer.SetVerticalScrollBarVisibility(grid, ScrollBarVisibility.Visible);

                    vsb.ClearValue(ScrollBar.ViewportSizeProperty);
                    vsb.ClearValue(ScrollBar.ValueProperty);
                    vsb.ClearValue(ScrollBar.MaximumProperty);

                    vsb.Minimum = 0;
                    vsb.SmallChange = 1;
                    vsb.SetBinding(ScrollBar.MaximumProperty, new Binding("DataView.PageCount") { Source = source });

                    vsb.Scroll -= Scroll;
                    vsb.Scroll += Scroll;
                }

                var maximumItems = (int)Math.Round(sw.RenderSize.Height / grid.RowHeight);

                if (maximumItems > source.PageSize)
                {
                    source.PageSize = maximumItems;
                }
            }
        }

        static void Scroll(object sender, ScrollEventArgs e)
        {
            var vsb = (ScrollBar)sender;
            var grid = vsb.ParentOfType<GridViewDataControl>();
            var source = CustomScrolling.GetDomainDataSource(grid);

            var timer = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(1000) };
            timer.Tick += (s, args) => 
            {
                timer.Stop();

                if (source.CanLoad)
                {
                    var pageIndex = (int)Math.Round(vsb.Value);

                    if (pageIndex != source.DataView.PageIndex)
                    {
                        source.DataView.MoveToPage(pageIndex);
                    }
                }
            };

            timer.Start();
        }
    }
}
